from __future__ import annotations

from digitalhub.entities.trigger._base.status import TriggerStatus


class TriggerStatusScheduler(TriggerStatus):
    """
    TriggerStatusScheduler status.
    """
